/**
 * 
 *   Copyright (c) 2010-2015, Csaba Debreceni, Istvan Rath and Daniel Varro
 *   All rights reserved. This program and the accompanying materials
 *   are made available under the terms of the Eclipse Public License v1.0
 *   which accompanies this distribution, and is available at
 *   http://www.eclipse.org/legal/epl-v10.html
 *  
 *   Contributors:
 *     Csaba Debreceni - initial API and implementation
 *  
 */
package org.eclipse.viatra.addon.viewers.runtime.model.patterns;

import java.util.Collection;
import java.util.HashSet;
import java.util.Set;
import org.apache.log4j.Logger;
import org.eclipse.viatra.addon.viewers.runtime.model.patterns.Param2containmentMatch;
import org.eclipse.viatra.addon.viewers.runtime.model.patterns.util.Param2containmentQuerySpecification;
import org.eclipse.viatra.addon.viewers.runtime.notation.Containment;
import org.eclipse.viatra.query.runtime.api.IMatchProcessor;
import org.eclipse.viatra.query.runtime.api.IQuerySpecification;
import org.eclipse.viatra.query.runtime.api.ViatraQueryEngine;
import org.eclipse.viatra.query.runtime.api.impl.BaseMatcher;
import org.eclipse.viatra.query.runtime.exception.ViatraQueryException;
import org.eclipse.viatra.query.runtime.matchers.tuple.Tuple;
import org.eclipse.viatra.query.runtime.util.ViatraQueryLoggingUtil;
import org.eclipse.viatra.transformation.views.traceability.Trace;

/**
 * Generated pattern matcher API of the org.eclipse.viatra.addon.viewers.runtime.model.patterns.param2containment pattern,
 * providing pattern-specific query methods.
 * 
 * <p>Use the pattern matcher on a given model via {@link #on(ViatraQueryEngine)},
 * e.g. in conjunction with {@link ViatraQueryEngine#on(Notifier)}.
 * 
 * <p>Matches of the pattern will be represented as {@link Param2containmentMatch}.
 * 
 * <p>Original source:
 * <code><pre>
 * {@literal @}QueryExplorer(checked = false)
 * pattern param2containment(source, target, trace, containment : Containment) {
 * 	Trace.params(trace, source);
 * 	Trace.params(trace, target);
 * 	Trace.targets(trace, containment);
 * } or {
 * 	Trace.objects(trace, source);
 * 	Trace.params(trace, target);
 * 	Trace.targets(trace, containment);
 * } or {
 * 	Trace.params(trace, source);
 * 	Trace.objects(trace, target);
 * 	Trace.targets(trace, containment);
 * } or {
 * 	Trace.objects(trace, source);
 * 	Trace.objects(trace, target);
 * 	Trace.targets(trace, containment);
 * }
 * </pre></code>
 * 
 * @see Param2containmentMatch
 * @see Param2containmentProcessor
 * @see Param2containmentQuerySpecification
 * 
 */
@SuppressWarnings("all")
public class Param2containmentMatcher extends BaseMatcher<Param2containmentMatch> {
  /**
   * Initializes the pattern matcher within an existing VIATRA Query engine.
   * If the pattern matcher is already constructed in the engine, only a light-weight reference is returned.
   * The match set will be incrementally refreshed upon updates.
   * @param engine the existing VIATRA Query engine in which this matcher will be created.
   * @throws ViatraQueryException if an error occurs during pattern matcher creation
   * 
   */
  public static Param2containmentMatcher on(final ViatraQueryEngine engine) throws ViatraQueryException {
    // check if matcher already exists
    Param2containmentMatcher matcher = engine.getExistingMatcher(querySpecification());
    if (matcher == null) {
    	matcher = (Param2containmentMatcher)engine.getMatcher(querySpecification());
    }
    return matcher;
  }
  
  /**
   * @throws ViatraQueryException if an error occurs during pattern matcher creation
   * @return an initialized matcher
   * @noreference This method is for internal matcher initialization by the framework, do not call it manually.
   * 
   */
  public static Param2containmentMatcher create() throws ViatraQueryException {
    return new Param2containmentMatcher();
  }
  
  private final static int POSITION_SOURCE = 0;
  
  private final static int POSITION_TARGET = 1;
  
  private final static int POSITION_TRACE = 2;
  
  private final static int POSITION_CONTAINMENT = 3;
  
  private final static Logger LOGGER = ViatraQueryLoggingUtil.getLogger(Param2containmentMatcher.class);
  
  /**
   * Initializes the pattern matcher within an existing VIATRA Query engine.
   * If the pattern matcher is already constructed in the engine, only a light-weight reference is returned.
   * The match set will be incrementally refreshed upon updates.
   * @param engine the existing VIATRA Query engine in which this matcher will be created.
   * @throws ViatraQueryException if an error occurs during pattern matcher creation
   * 
   */
  private Param2containmentMatcher() throws ViatraQueryException {
    super(querySpecification());
  }
  
  /**
   * Returns the set of all matches of the pattern that conform to the given fixed values of some parameters.
   * @param pSource the fixed value of pattern parameter source, or null if not bound.
   * @param pTarget the fixed value of pattern parameter target, or null if not bound.
   * @param pTrace the fixed value of pattern parameter trace, or null if not bound.
   * @param pContainment the fixed value of pattern parameter containment, or null if not bound.
   * @return matches represented as a Param2containmentMatch object.
   * 
   */
  public Collection<Param2containmentMatch> getAllMatches(final Object pSource, final Object pTarget, final Trace pTrace, final Containment pContainment) {
    return rawGetAllMatches(new Object[]{pSource, pTarget, pTrace, pContainment});
  }
  
  /**
   * Returns an arbitrarily chosen match of the pattern that conforms to the given fixed values of some parameters.
   * Neither determinism nor randomness of selection is guaranteed.
   * @param pSource the fixed value of pattern parameter source, or null if not bound.
   * @param pTarget the fixed value of pattern parameter target, or null if not bound.
   * @param pTrace the fixed value of pattern parameter trace, or null if not bound.
   * @param pContainment the fixed value of pattern parameter containment, or null if not bound.
   * @return a match represented as a Param2containmentMatch object, or null if no match is found.
   * 
   */
  public Param2containmentMatch getOneArbitraryMatch(final Object pSource, final Object pTarget, final Trace pTrace, final Containment pContainment) {
    return rawGetOneArbitraryMatch(new Object[]{pSource, pTarget, pTrace, pContainment});
  }
  
  /**
   * Indicates whether the given combination of specified pattern parameters constitute a valid pattern match,
   * under any possible substitution of the unspecified parameters (if any).
   * @param pSource the fixed value of pattern parameter source, or null if not bound.
   * @param pTarget the fixed value of pattern parameter target, or null if not bound.
   * @param pTrace the fixed value of pattern parameter trace, or null if not bound.
   * @param pContainment the fixed value of pattern parameter containment, or null if not bound.
   * @return true if the input is a valid (partial) match of the pattern.
   * 
   */
  public boolean hasMatch(final Object pSource, final Object pTarget, final Trace pTrace, final Containment pContainment) {
    return rawHasMatch(new Object[]{pSource, pTarget, pTrace, pContainment});
  }
  
  /**
   * Returns the number of all matches of the pattern that conform to the given fixed values of some parameters.
   * @param pSource the fixed value of pattern parameter source, or null if not bound.
   * @param pTarget the fixed value of pattern parameter target, or null if not bound.
   * @param pTrace the fixed value of pattern parameter trace, or null if not bound.
   * @param pContainment the fixed value of pattern parameter containment, or null if not bound.
   * @return the number of pattern matches found.
   * 
   */
  public int countMatches(final Object pSource, final Object pTarget, final Trace pTrace, final Containment pContainment) {
    return rawCountMatches(new Object[]{pSource, pTarget, pTrace, pContainment});
  }
  
  /**
   * Executes the given processor on each match of the pattern that conforms to the given fixed values of some parameters.
   * @param pSource the fixed value of pattern parameter source, or null if not bound.
   * @param pTarget the fixed value of pattern parameter target, or null if not bound.
   * @param pTrace the fixed value of pattern parameter trace, or null if not bound.
   * @param pContainment the fixed value of pattern parameter containment, or null if not bound.
   * @param processor the action that will process each pattern match.
   * 
   */
  public void forEachMatch(final Object pSource, final Object pTarget, final Trace pTrace, final Containment pContainment, final IMatchProcessor<? super Param2containmentMatch> processor) {
    rawForEachMatch(new Object[]{pSource, pTarget, pTrace, pContainment}, processor);
  }
  
  /**
   * Executes the given processor on an arbitrarily chosen match of the pattern that conforms to the given fixed values of some parameters.
   * Neither determinism nor randomness of selection is guaranteed.
   * @param pSource the fixed value of pattern parameter source, or null if not bound.
   * @param pTarget the fixed value of pattern parameter target, or null if not bound.
   * @param pTrace the fixed value of pattern parameter trace, or null if not bound.
   * @param pContainment the fixed value of pattern parameter containment, or null if not bound.
   * @param processor the action that will process the selected match.
   * @return true if the pattern has at least one match with the given parameter values, false if the processor was not invoked
   * 
   */
  public boolean forOneArbitraryMatch(final Object pSource, final Object pTarget, final Trace pTrace, final Containment pContainment, final IMatchProcessor<? super Param2containmentMatch> processor) {
    return rawForOneArbitraryMatch(new Object[]{pSource, pTarget, pTrace, pContainment}, processor);
  }
  
  /**
   * Returns a new (partial) match.
   * This can be used e.g. to call the matcher with a partial match.
   * <p>The returned match will be immutable. Use {@link #newEmptyMatch()} to obtain a mutable match object.
   * @param pSource the fixed value of pattern parameter source, or null if not bound.
   * @param pTarget the fixed value of pattern parameter target, or null if not bound.
   * @param pTrace the fixed value of pattern parameter trace, or null if not bound.
   * @param pContainment the fixed value of pattern parameter containment, or null if not bound.
   * @return the (partial) match object.
   * 
   */
  public Param2containmentMatch newMatch(final Object pSource, final Object pTarget, final Trace pTrace, final Containment pContainment) {
    return Param2containmentMatch.newMatch(pSource, pTarget, pTrace, pContainment);
  }
  
  /**
   * Retrieve the set of values that occur in matches for source.
   * @return the Set of all values, null if no parameter with the given name exists, empty set if there are no matches
   * 
   */
  protected Set<Object> rawAccumulateAllValuesOfsource(final Object[] parameters) {
    Set<Object> results = new HashSet<Object>();
    rawAccumulateAllValues(POSITION_SOURCE, parameters, results);
    return results;
  }
  
  /**
   * Retrieve the set of values that occur in matches for source.
   * @return the Set of all values, null if no parameter with the given name exists, empty set if there are no matches
   * 
   */
  public Set<Object> getAllValuesOfsource() {
    return rawAccumulateAllValuesOfsource(emptyArray());
  }
  
  /**
   * Retrieve the set of values that occur in matches for source.
   * @return the Set of all values, null if no parameter with the given name exists, empty set if there are no matches
   * 
   */
  public Set<Object> getAllValuesOfsource(final Param2containmentMatch partialMatch) {
    return rawAccumulateAllValuesOfsource(partialMatch.toArray());
  }
  
  /**
   * Retrieve the set of values that occur in matches for source.
   * @return the Set of all values, null if no parameter with the given name exists, empty set if there are no matches
   * 
   */
  public Set<Object> getAllValuesOfsource(final Object pTarget, final Trace pTrace, final Containment pContainment) {
    return rawAccumulateAllValuesOfsource(new Object[]{
    null, 
    pTarget, 
    pTrace, 
    pContainment
    });
  }
  
  /**
   * Retrieve the set of values that occur in matches for target.
   * @return the Set of all values, null if no parameter with the given name exists, empty set if there are no matches
   * 
   */
  protected Set<Object> rawAccumulateAllValuesOftarget(final Object[] parameters) {
    Set<Object> results = new HashSet<Object>();
    rawAccumulateAllValues(POSITION_TARGET, parameters, results);
    return results;
  }
  
  /**
   * Retrieve the set of values that occur in matches for target.
   * @return the Set of all values, null if no parameter with the given name exists, empty set if there are no matches
   * 
   */
  public Set<Object> getAllValuesOftarget() {
    return rawAccumulateAllValuesOftarget(emptyArray());
  }
  
  /**
   * Retrieve the set of values that occur in matches for target.
   * @return the Set of all values, null if no parameter with the given name exists, empty set if there are no matches
   * 
   */
  public Set<Object> getAllValuesOftarget(final Param2containmentMatch partialMatch) {
    return rawAccumulateAllValuesOftarget(partialMatch.toArray());
  }
  
  /**
   * Retrieve the set of values that occur in matches for target.
   * @return the Set of all values, null if no parameter with the given name exists, empty set if there are no matches
   * 
   */
  public Set<Object> getAllValuesOftarget(final Object pSource, final Trace pTrace, final Containment pContainment) {
    return rawAccumulateAllValuesOftarget(new Object[]{
    pSource, 
    null, 
    pTrace, 
    pContainment
    });
  }
  
  /**
   * Retrieve the set of values that occur in matches for trace.
   * @return the Set of all values, null if no parameter with the given name exists, empty set if there are no matches
   * 
   */
  protected Set<Trace> rawAccumulateAllValuesOftrace(final Object[] parameters) {
    Set<Trace> results = new HashSet<Trace>();
    rawAccumulateAllValues(POSITION_TRACE, parameters, results);
    return results;
  }
  
  /**
   * Retrieve the set of values that occur in matches for trace.
   * @return the Set of all values, null if no parameter with the given name exists, empty set if there are no matches
   * 
   */
  public Set<Trace> getAllValuesOftrace() {
    return rawAccumulateAllValuesOftrace(emptyArray());
  }
  
  /**
   * Retrieve the set of values that occur in matches for trace.
   * @return the Set of all values, null if no parameter with the given name exists, empty set if there are no matches
   * 
   */
  public Set<Trace> getAllValuesOftrace(final Param2containmentMatch partialMatch) {
    return rawAccumulateAllValuesOftrace(partialMatch.toArray());
  }
  
  /**
   * Retrieve the set of values that occur in matches for trace.
   * @return the Set of all values, null if no parameter with the given name exists, empty set if there are no matches
   * 
   */
  public Set<Trace> getAllValuesOftrace(final Object pSource, final Object pTarget, final Containment pContainment) {
    return rawAccumulateAllValuesOftrace(new Object[]{
    pSource, 
    pTarget, 
    null, 
    pContainment
    });
  }
  
  /**
   * Retrieve the set of values that occur in matches for containment.
   * @return the Set of all values, null if no parameter with the given name exists, empty set if there are no matches
   * 
   */
  protected Set<Containment> rawAccumulateAllValuesOfcontainment(final Object[] parameters) {
    Set<Containment> results = new HashSet<Containment>();
    rawAccumulateAllValues(POSITION_CONTAINMENT, parameters, results);
    return results;
  }
  
  /**
   * Retrieve the set of values that occur in matches for containment.
   * @return the Set of all values, null if no parameter with the given name exists, empty set if there are no matches
   * 
   */
  public Set<Containment> getAllValuesOfcontainment() {
    return rawAccumulateAllValuesOfcontainment(emptyArray());
  }
  
  /**
   * Retrieve the set of values that occur in matches for containment.
   * @return the Set of all values, null if no parameter with the given name exists, empty set if there are no matches
   * 
   */
  public Set<Containment> getAllValuesOfcontainment(final Param2containmentMatch partialMatch) {
    return rawAccumulateAllValuesOfcontainment(partialMatch.toArray());
  }
  
  /**
   * Retrieve the set of values that occur in matches for containment.
   * @return the Set of all values, null if no parameter with the given name exists, empty set if there are no matches
   * 
   */
  public Set<Containment> getAllValuesOfcontainment(final Object pSource, final Object pTarget, final Trace pTrace) {
    return rawAccumulateAllValuesOfcontainment(new Object[]{
    pSource, 
    pTarget, 
    pTrace, 
    null
    });
  }
  
  @Override
  protected Param2containmentMatch tupleToMatch(final Tuple t) {
    try {
    	return Param2containmentMatch.newMatch((Object) t.get(POSITION_SOURCE), (Object) t.get(POSITION_TARGET), (Trace) t.get(POSITION_TRACE), (Containment) t.get(POSITION_CONTAINMENT));
    } catch(ClassCastException e) {
    	LOGGER.error("Element(s) in tuple not properly typed!",e);
    	return null;
    }
  }
  
  @Override
  protected Param2containmentMatch arrayToMatch(final Object[] match) {
    try {
    	return Param2containmentMatch.newMatch((Object) match[POSITION_SOURCE], (Object) match[POSITION_TARGET], (Trace) match[POSITION_TRACE], (Containment) match[POSITION_CONTAINMENT]);
    } catch(ClassCastException e) {
    	LOGGER.error("Element(s) in array not properly typed!",e);
    	return null;
    }
  }
  
  @Override
  protected Param2containmentMatch arrayToMatchMutable(final Object[] match) {
    try {
    	return Param2containmentMatch.newMutableMatch((Object) match[POSITION_SOURCE], (Object) match[POSITION_TARGET], (Trace) match[POSITION_TRACE], (Containment) match[POSITION_CONTAINMENT]);
    } catch(ClassCastException e) {
    	LOGGER.error("Element(s) in array not properly typed!",e);
    	return null;
    }
  }
  
  /**
   * @return the singleton instance of the query specification of this pattern
   * @throws ViatraQueryException if the pattern definition could not be loaded
   * 
   */
  public static IQuerySpecification<Param2containmentMatcher> querySpecification() throws ViatraQueryException {
    return Param2containmentQuerySpecification.instance();
  }
}
