/**
 * Copyright (c) 2010-2016, Abel Hegedus, IncQuery Labs Ltd.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *   Abel Hegedus - initial API and implementation
 */
package org.eclipse.viatra.query.tooling.ui.queryresult.util;

import com.google.common.collect.Iterables;
import java.util.Collection;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;
import org.eclipse.emf.common.notify.AdapterFactory;
import org.eclipse.emf.edit.provider.ComposedAdapterFactory;
import org.eclipse.emf.edit.provider.ReflectiveItemProviderAdapterFactory;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.viatra.query.runtime.registry.IQuerySpecificationRegistryEntry;
import org.eclipse.viatra.query.tooling.ui.queryregistry.QueryRegistryTreeEntry;
import org.eclipse.viatra.query.tooling.ui.queryregistry.QueryRegistryTreePackage;
import org.eclipse.viatra.query.tooling.ui.queryregistry.QueryRegistryTreeSource;
import org.eclipse.xtext.xbase.lib.CollectionLiterals;
import org.eclipse.xtext.xbase.lib.Functions.Function1;
import org.eclipse.xtext.xbase.lib.IterableExtensions;
import org.eclipse.xtext.xbase.lib.IteratorExtensions;
import org.eclipse.xtext.xbase.lib.Procedures.Procedure1;

/**
 * @author Abel Hegedus
 */
@SuppressWarnings("all")
public class QueryResultViewUtil {
  /**
   * Constructor hidden for utility class
   */
  protected QueryResultViewUtil() {
  }
  
  public static Set<QueryRegistryTreeEntry> getRegistryEntriesFromSelection(final IStructuredSelection selection) {
    final HashSet<QueryRegistryTreeEntry> selectedQueries = CollectionLiterals.<QueryRegistryTreeEntry>newHashSet();
    Iterator _iterator = selection.iterator();
    final Procedure1<Object> _function = new Procedure1<Object>() {
      @Override
      public void apply(final Object it) {
        boolean _matched = false;
        if (!_matched) {
          if (it instanceof QueryRegistryTreeEntry) {
            _matched=true;
            selectedQueries.add(((QueryRegistryTreeEntry)it));
          }
        }
        if (!_matched) {
          if (it instanceof QueryRegistryTreePackage) {
            _matched=true;
            Map<String, QueryRegistryTreeEntry> _entries = ((QueryRegistryTreePackage)it).getEntries();
            Collection<QueryRegistryTreeEntry> _values = _entries.values();
            selectedQueries.addAll(_values);
          }
        }
        if (!_matched) {
          if (it instanceof QueryRegistryTreeSource) {
            _matched=true;
            Map<String, QueryRegistryTreePackage> _packages = ((QueryRegistryTreeSource)it).getPackages();
            Collection<QueryRegistryTreePackage> _values = _packages.values();
            final Function1<QueryRegistryTreePackage, Collection<QueryRegistryTreeEntry>> _function = new Function1<QueryRegistryTreePackage, Collection<QueryRegistryTreeEntry>>() {
              @Override
              public Collection<QueryRegistryTreeEntry> apply(final QueryRegistryTreePackage it) {
                Map<String, QueryRegistryTreeEntry> _entries = it.getEntries();
                return _entries.values();
              }
            };
            Iterable<Collection<QueryRegistryTreeEntry>> _map = IterableExtensions.<QueryRegistryTreePackage, Collection<QueryRegistryTreeEntry>>map(_values, _function);
            Iterable<QueryRegistryTreeEntry> _flatten = Iterables.<QueryRegistryTreeEntry>concat(_map);
            Iterables.<QueryRegistryTreeEntry>addAll(selectedQueries, _flatten);
          }
        }
      }
    };
    IteratorExtensions.<Object>forEach(_iterator, _function);
    return selectedQueries;
  }
  
  public static Iterable<IQuerySpecificationRegistryEntry> unwrapEntries(final Set<QueryRegistryTreeEntry> entries) {
    final Function1<QueryRegistryTreeEntry, IQuerySpecificationRegistryEntry> _function = new Function1<QueryRegistryTreeEntry, IQuerySpecificationRegistryEntry>() {
      @Override
      public IQuerySpecificationRegistryEntry apply(final QueryRegistryTreeEntry it) {
        return it.getEntry();
      }
    };
    return IterableExtensions.<QueryRegistryTreeEntry, IQuerySpecificationRegistryEntry>map(entries, _function);
  }
  
  /**
   * @since 1.4
   */
  public static AdapterFactory getGenericAdapterFactory() {
    final ComposedAdapterFactory adapterFactory = new ComposedAdapterFactory(ComposedAdapterFactory.Descriptor.Registry.INSTANCE);
    ReflectiveItemProviderAdapterFactory _reflectiveItemProviderAdapterFactory = new ReflectiveItemProviderAdapterFactory();
    adapterFactory.addAdapterFactory(_reflectiveItemProviderAdapterFactory);
    return adapterFactory;
  }
}
